#!perl -w
use Test::More;
use strict;
use DE_EPAGES::Core::API::Html qw(
    BalanceHTMLTags
    RemoveHtmlTags
    CloseOpenHTMLTags
    ReplaceURIWithURL
);

plan tests => 24;

## no critic qw(ProhibitLongLines)

my $SourceText  = '<p><h1><span color="#333311">tsteawt</span><strong><!-- pagebreak --></strong></h1><p>dfasdasdasdaasasfasfs</p>asdasda';
my $GoalText    = '<p></p><h1><span color="#333311">tsteawt</span><strong><!-- pagebreak --></strong></h1><p>dfasdasdasdaasasfasfs</p>asdasda';
cmp_ok(BalanceHTMLTags($SourceText), 'eq', $GoalText, 'BalanceHTMLTags headings and paragraphs');

$SourceText  = '<p><table><tr><td>test</tr><tr><td><div><strong><!-- pagebreak -->';
$GoalText    = '<p></p><table><tr><td>test</td></tr><tr><td><div><strong><!-- pagebreak --></strong></div></td></tr></table>';
cmp_ok(BalanceHTMLTags($SourceText), 'eq', $GoalText, 'BalanceHTMLTags malformed table');

$SourceText  = '<p>&nbsp;</p><h2></h2><h1>tsteawt<b></b></h1><p>dfasdasdasdaasasfasf<table class="TinyMCETable" style="border: 1px solid #e27c7c; width: 100px; height: 50px; background-color: #703030;"><tbody><tr><td><font color="#e5d5d5"></font></td><td style="border: 1px solid #ff8400; text-align: center;"><font color="#e5d5d5">sdfsf</font></td></tr><tr><td></td><td><i>assaasas</i></td></tr></tbody></table>s</p><p>asdasda</p><p><i style="background-color: #e00808;"><span style="text-decoration: underline;"><a href="?ObjectPath=/Shops/DemoShop/Categories/CustomerInformation">Wichtig<table class=" TinyMCETable" style="border: 1px solid #302020; width: 20px; height: 30px; background-color: #bfffbf;"><tbody><tr><td></td><td>asd</td></tr><tr><td></td><td>asdddasas<!-- pagebreak -->';
$GoalText    = '<p>&nbsp;</p><h2></h2><h1>tsteawt<b></b></h1><p>dfasdasdasdaasasfasf</p><table class="TinyMCETable" style="border: 1px solid #e27c7c; width: 100px; height: 50px; background-color: #703030;"><tbody><tr><td><font color="#e5d5d5"></font></td><td style="border: 1px solid #ff8400; text-align: center;"><font color="#e5d5d5">sdfsf</font></td></tr><tr><td></td><td><i>assaasas</i></td></tr></tbody></table>s<p>asdasda</p><p><i style="background-color: #e00808;"><span style="text-decoration: underline;"><a href="?ObjectPath=/Shops/DemoShop/Categories/CustomerInformation">Wichtig</a></span></i></p><table class=" TinyMCETable" style="border: 1px solid #302020; width: 20px; height: 30px; background-color: #bfffbf;"><tbody><tr><td></td><td>asd</td></tr><tr><td></td><td>asdddasas<!-- pagebreak --></td></tr></tbody></table>';
cmp_ok(BalanceHTMLTags($SourceText), 'eq', $GoalText, 'BalanceHTMLTags complex from tinymce');

$SourceText = '<p class=test"><span style="test2"></p>';
$GoalText   = '<p class="test&quot;"><span style="test2"></span></p>';
cmp_ok(BalanceHTMLTags($SourceText), 'eq', $GoalText, 'BalanceHTMLTags missing quotes at the beginning');

$SourceText = '<script type="text/javascript">jQuery("<script>")</script>';
$GoalText   = '<script type="text/javascript">jQuery("<script>")</script>';
cmp_ok(BalanceHTMLTags($SourceText), 'eq', $GoalText, 'BalanceHTMLTags with script tags');

$SourceText  = '<p><h1><span color="#333311">tsteawt</span><strong><!-- pagebreak --></strong></h1><p>dfasdasdasdaasasfasfs</p>asdasda';
$GoalText    = '<p></p><h1><span color="#333311">tsteawt</span><strong><!-- pagebreak --></strong></h1><p>dfasdasdasdaasasfasfs</p>asdasda';
cmp_ok(BalanceHTMLTags($SourceText), 'eq', $GoalText, 'BalanceHTMLTags headings and paragraphs');


# CloseOpenHTMLTags
$SourceText  = '<p><h1><span color="#333311">tsteawt</span><strong><!-- pagebreak --></strong></h1><p>dfasdasdasdaasasfasfs</p>asdasda';
$GoalText    = '<p><h1><span color="#333311">tsteawt</span><strong><!-- pagebreak --></strong></h1></p><p>dfasdasdasdaasasfasfs</p>asdasda';
cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'CloseOpenHTMLTags headings and paragraphs');

$SourceText  = '<p><table><tr><td>test</tr><tr><td><div><strong><!-- pagebreak -->';
$GoalText    = '<p><table><tr><td>test</td></tr><tr><td><div><strong><!-- pagebreak --></strong></div></td></tr></table></p>';
cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'CloseOpenHTMLTags malformed table');

$SourceText  = '<p>&nbsp;</p><h2></h2><h1>tsteawt<b></b></h1><p>dfasdasdasdaasasfasf<table class="TinyMCETable" style="border: 1px solid #e27c7c; width: 100px; height: 50px; background-color: #703030;"><tbody><tr><td><font color="#e5d5d5"></font></td><td style="border: 1px solid #ff8400; text-align: center;"><font color="#e5d5d5">sdfsf</font></td></tr><tr><td></td><td><i>assaasas</i></td></tr></tbody></table>s</p><p>asdasda</p><p><i style="background-color: #e00808;"><span style="text-decoration: underline;"><a href="?ObjectPath=/Shops/DemoShop/Categories/CustomerInformation">Wichtig<table class=" TinyMCETable" style="border: 1px solid #302020; width: 20px; height: 30px; background-color: #bfffbf;"><tbody><tr><td></td><td>asd</td></tr><tr><td></td><td>asdddasas<!-- pagebreak -->';
$GoalText    = '<p>&nbsp;</p><h2></h2><h1>tsteawt<b></b></h1><p>dfasdasdasdaasasfasf<table class="TinyMCETable" style="border: 1px solid #e27c7c; width: 100px; height: 50px; background-color: #703030;"><tbody><tr><td><font color="#e5d5d5"></font></td><td style="border: 1px solid #ff8400; text-align: center;"><font color="#e5d5d5">sdfsf</font></td></tr><tr><td></td><td><i>assaasas</i></td></tr></tbody></table>s</p><p>asdasda</p><p><i style="background-color: #e00808;"><span style="text-decoration: underline;"><a href="?ObjectPath=/Shops/DemoShop/Categories/CustomerInformation">Wichtig<table class=" TinyMCETable" style="border: 1px solid #302020; width: 20px; height: 30px; background-color: #bfffbf;"><tbody><tr><td></td><td>asd</td></tr><tr><td></td><td>asdddasas<!-- pagebreak --></td></tr></tbody></table></a></span></i></p>';
cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'CloseOpenHTMLTags complex from tinymce');

$SourceText = '<p class="test><span style="test2"></p>';
$GoalText   = '<p class="test><span style="test2"></span></p>';
cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'CloseOpenHTMLTags missing quotes at the end');

$SourceText = '<p class="test test2 id="test><span style="test2"></p>';
$GoalText   = '<p class="test test2 id="test><span style="test2"></span></p>';
cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'CloseOpenHTMLTags missing quotes with more values');

$SourceText = '<p class=test"><span style="test2"></p>';
$GoalText   = '<p class=test"><span style="test2"></span></p>';
cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'CloseOpenHTMLTags missing quotes at the beginning');

$SourceText = '<script type="text/javascript">jQuery("<script>")</script>';
$GoalText   = '<script type="text/javascript">jQuery("<script>")</script>';
cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'CloseOpenHTMLTags with script tags');

# javascript event in html
$SourceText = '<a href="?" onclick="openWindow(this.href+\'&amp;ViewAction=ViewPopup\',\'\',\'PopUp\');">Test</a>';
$GoalText = '<a href="?" onclick="openWindow(this.href+\'&amp;ViewAction=ViewPopup\',\'\',\'PopUp\');">Test</a>';
cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'CloseOpenHTMLTags inline javascript on dom node');

# javascript event in html
$SourceText = 'Ich habe die <a href="?ObjectPath=#Shop.Path[url]/Categories/TermsAndConditions" onclick="openWindow(this.href+\'&amp;ViewAction=ViewPopup\',\'\',\'PopUp\');return false;">Allgemeinen Geschäftsbedingungen</a> gelesen und erkenne sie hiermit an.<!-- comment! -->';
$GoalText = 'Ich habe die <a href="?ObjectPath=#Shop.Path[url]/Categories/TermsAndConditions" onclick="openWindow(this.href+\'&amp;ViewAction=ViewPopup\',\'\',\'PopUp\');return false;">Allgemeinen Geschäftsbedingungen</a> gelesen und erkenne sie hiermit an.<!-- comment! -->';
cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'CloseOpenHTMLTags inline javascript on dom node');

# javascript event in html
$SourceText = '/a_s_d/!#Hündchen Ölgemälde-ÉÊËÌÍ#{}(b+).создавать<a href="/a_s_d/!#Hündchen Ölgemälde-ÉÊËÌÍ#{}(b+).создавать?ObjectPath=#Shop.Path[url]/Categories/TermsAndConditions" onclick="openWindow(this.href+\'&amp;ViewAction=ViewPopup\',\'\',\'PopUp\');return false;">Allgemeinen Geschäftsbedingungen</a> gelesen und erkenne sie hiermit an.<!-- comment! -->';
$GoalText = '/a_s_d/!#Hündchen Ölgemälde-ÉÊËÌÍ#{}(b+).создавать<a href="/a_s_d/!#Hündchen Ölgemälde-ÉÊËÌÍ#{}(b+).создавать?ObjectPath=#Shop.Path[url]/Categories/TermsAndConditions" onclick="openWindow(this.href+\'&amp;ViewAction=ViewPopup\',\'\',\'PopUp\');return false;">Allgemeinen Geschäftsbedingungen</a> gelesen und erkenne sie hiermit an.<!-- comment! -->';
cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'CloseOpenHTMLTags inline javascript on dom node');

$SourceText = 'Обратите внимание на наши <div><a rel="nofollow" href="?ObjectPath=Categories/CustomerInformation&amp;amp;ViewAction=ViewPopup" target="MyPopup">правила отказа</a> и <a rel="nofollow" href="?ObjectPath=Categories/TermsAndConditions&amp;amp;ViewAction=ViewPopup" target="MyPopup">общие условия</a>.';
$GoalText = 'Обратите внимание на наши <div><a rel="nofollow" href="?ObjectPath=Categories/CustomerInformation&amp;amp;ViewAction=ViewPopup" target="MyPopup">правила отказа</a> и <a rel="nofollow" href="?ObjectPath=Categories/TermsAndConditions&amp;amp;ViewAction=ViewPopup" target="MyPopup">общие условия</a>.</div>';
cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'CloseOpenHTMLTags russian text');

$SourceText = '<script type="text/javascript">jQuery("<div>")</script>';
$GoalText   = '<script type="text/javascript">jQuery("<div>")</script>';
cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'CloseOpenHTMLTags with javascript selector');



# RemoveHtmlTags
$SourceText  = <<'END_TEXT';
    <p>
      <h1> <span color="#333311">headline</span> <strong><!-- pagebreak --></strong></h1>
    <p>text1<br>text 2<hr />&nbsp;&nbsp;text3 &amp;</p>
    <script language="JScript">
      document.all.obsolete = 1;
    </script>
    <STYLE>
      document.all.obsolete = 1;
    </STYLE>
    fin.
END_TEXT
$GoalText = <<'END_TEXT';
headline
text1
text 2
  text3 &
fin.
END_TEXT
is( RemoveHtmlTags($SourceText)."\n", $GoalText, 'RemoveHtmlTags' );


my $BaseURL = 'http://example.de';

my $SourceText = '<div><a href="?Parameter=Value"><img src="WebRoot/test.png" /></a></div>';
my $GoalText = '<div><a href="http://example.de?Parameter=Value"><img src="http://example.de/WebRoot/test.png" /></a></div>';
is( ReplaceURIWithURL($SourceText, $BaseURL), $GoalText, 'replace uris with full url in a- and img-tags.' );

$SourceText = '<div><a href="http://example.de?Parameter=Value"><img src="WebRoot/test.png" /></a></div>';
$GoalText = '<div><a href="http://example.de?Parameter=Value"><img src="http://example.de/WebRoot/test.png" /></a></div>';
is( ReplaceURIWithURL($SourceText, $BaseURL), $GoalText, 'do not replace correct urls.' );

$BaseURL .= '/epages/ShopAlias.sf';
$SourceText = '<div><a href="?Parameter=Value"><img src="http://example.de/WebRoot/test.png" /></a></div>';
$GoalText = '<div><a href="http://example.de/epages/ShopAlias.sf?Parameter=Value"><img src="http://example.de/WebRoot/test.png" /></a></div>';
is( ReplaceURIWithURL($SourceText, $BaseURL), $GoalText, 'replace uris with full url even for extended baseurls.' );

$SourceText = '<script src="/blablabla.js"></script>';
$GoalText = '<script src="http://example.de/blablabla.js"></script>';
is( ReplaceURIWithURL($SourceText, $BaseURL), $GoalText, 'replace uris with full url in script sources.' );

$SourceText = '<a href="test.html"><img src="/WebRoot/test.png" /></a>';
$GoalText = '<a href="test.html"><img src="http://example.de/WebRoot/test.png" /></a>';
is( ReplaceURIWithURL($SourceText, $BaseURL, ['img']), $GoalText, 'replace uris with full url only for special tags.' );